Grub real mode BIOS issue
=========================

This is a stripped down version of the problem I'm having with GRUB.

In short, if you boot through BIOS, then everything works as expected. If you boot the same code through GRUB, switch back to real
mode, then some BIOS routines are acting strange.

Test Cases
----------
```
make qemurom
```
Boots stage2.bin from ROM, works as expected. Screen resolution changes, and it prints "End" on screen.

```
make qemubios
```
Boots stage2.bin through boot sector, works as expected. Screen resolution changes, and it prints "End" on screen.

```
make qemugrub
```
Boots stage2.bin through GRUB. It does not work. It sets a wrong resolution, and then it prints "nd" (missing first letter).

```
make bochsrom
```
Boots stage2.bin from ROM, works as expected. Screen resolution changes correctly (no output with this vgabios).

```
make bochsbios
```
Boots stage2.bin through boot sector, works as expected. Screen resolution changes correctly (no output with this vgabios).

```
make bochsgrub
```
Boots stage2.bin through GRUB. Does not work either. Screen resolution does not change, and it gets into an infinite loop
(not the expected loop). Bochs prints lots of errors:
```
02663554094e[DMA   ] io write to address 00000000, len=2
02663578409e[DMA   ] io write to address 00000000, len=2
02663602724e[DMA   ] io write to address 00000000, len=2
02663627039e[DMA   ] io write to address 00000000, len=2
02663651354e[DMA   ] io write to address 00000000, len=2
02663675669e[DMA   ] io write to address 00000000, len=2
02663699984e[DMA   ] io write to address 00000000, len=2
02663724299e[DMA   ] io write to address 00000000, len=2
02663748614e[DMA   ] io write to address 00000000, len=2
02663772929e[DMA   ] io write to address 00000000, len=2
02663797244e[DMA   ] io write to address 00000000, len=2
```
and sometimes
```
02665430999e[DMA   ] DMA-1: read of temporary register always returns 0
02665433976e[DMA   ] DMA-1: read of temporary register always returns 0
```
When I press close the window, I see this:
```
02670210184e[DMA   ] io write to address 00000000, len=2
02670234499e[DMA   ] io write to address 00000000, len=2
02670248000p[XGUI  ] >>PANIC<< POWER button turned off.
========================================================================
Bochs is exiting with the following message:
[XGUI  ] POWER button turned off.
========================================================================
02670248000i[CPU0  ] CPU is in real mode (active)
02670248000i[CPU0  ] CS.mode = 16 bit
02670248000i[CPU0  ] SS.mode = 32 bit
02670248000i[CPU0  ] EFER   = 0x00000000
02670248000i[CPU0  ] | EAX=6000003f  EBX=00010000  ECX=00000003  EDX=0000003f
02670248000i[CPU0  ] | ESP=00074dc2  EBP=00004dc6  ESI=00000a46  EDI=0000a000
02670248000i[CPU0  ] | IOPL=0 id vip vif ac vm rf nt of df if tf SF zf af pf CF
02670248000i[CPU0  ] | SEG sltr(index|ti|rpl)     base    limit G D
02670248000i[CPU0  ] |  CS:c000( 0002| 0|  0) 000c0000 0000ffff 0 0
02670248000i[CPU0  ] |  DS:c000( 0003| 0|  0) 000c0000 ffffffff 1 1
02670248000i[CPU0  ] |  SS:0000( 0003| 0|  0) 00000000 ffffffff 1 1
02670248000i[CPU0  ] |  ES:0000( 0003| 0|  0) 00000000 ffffffff 1 1
02670248000i[CPU0  ] |  FS:0018( 0003| 0|  0) 00000000 ffffffff 1 1
02670248000i[CPU0  ] |  GS:0018( 0003| 0|  0) 00000000 ffffffff 1 1
02670248000i[CPU0  ] | EIP=00003c75 (00003c75)
02670248000i[CPU0  ] | CR0=0x60000010 CR2=0x00000000
02670248000i[CPU0  ] | CR3=0x00000000 CR4=0x00000000
(0).[2670248000] [0x0000000c3c17] c000:3c17 (unk. ctxt): jb .+92 (0x000c3c75)      ; 725c
02670248000i[CMOS  ] Last time is 1578921467 (Mon Jan 13 14:17:47 2020)
02670248000i[XGUI  ] Exit
```
So the VM is executing BIOS ROM code. In the orginal project, not in this stripped down PoC, the infinite loop happens at
address F000h:E871h (which is also BIOS code). Ocassionally the BIOS code does not freeze, and code stops at 8E0h as expected,
however the VESA mode is not set as it should.

Just for completeness, I've also tested my CDROM image (not created by grub-mkrescue) with both qemu and bochs, and there are
no "io write to address 000000" errors, both works as expected. So we can rule out CDROM as the source of the problem. The
problem is clearly only GRUB related.

Why go BIOS routines haywire when I switch back to real mode from Multiboot? Is it possible that GRUB destroys some important
BIOS variables in BDA or EBDA? Segment registers, IVT, all other CPU related configuration looks good.

The Question
------------

How could one restore real mode properly so that BIOS can be used again without problems?
